from rich.console import Console

# Create a console instance for logging
console = Console()


def log_info(msg: str) -> None:
    """Logs an informational message to the console."""
    console.print(f"[blue]{msg}[/blue]")


def log_success(msg: str) -> None:
    """Logs a success message to the console."""
    console.print(f"[green]{msg}[/green]")


def log_error(msg: str) -> None:
    """Logs an error message to the console."""
    console.print(f"[red]{msg}[/red]")


def log_warn(msg: str) -> None:
    """Logs a warning message to the console."""
    console.print(f"[yellow]{msg}[/yellow]")



import os
import sys
import time
from typing import Any

from watchdog.events import FileSystemEvent, FileSystemEventHandler
from watchdog.observers import Observer
from watchdog.observers.api import BaseObserver

from autofmt.services.get_relative_path import get_relative_path
from autofmt.services.formatter import run_formatters
from autofmt.services.logger import log_info


class FormatHandler(FileSystemEventHandler):
    def __init__(self, config: dict[str, Any]) -> None:
        super().__init__()
        self.config = config

    def on_modified(self, event: FileSystemEvent) -> None:
        global privious_path
        if event.is_directory or not event.src_path.endswith(".py"):
            return
        filepath = os.path.abspath(event.src_path)
        relative_path = get_relative_path(filepath, self.config)
        log_info(f"\nDetected change:\n  {relative_path}")
        run_formatters(relative_path, self.config)


def run_watcher(config: dict[str, Any]) -> None:
    path: str = config.get("watch_path", ".")
    event_handler: FormatHandler = FormatHandler(config)
    observer: BaseObserver = Observer()
    observer.schedule(event_handler, path=path, recursive=True)  # type: ignore
    observer.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
        log_info("\nStopping watcher...")
        sys.stdout.write("\033[?25h\r")
    observer.join()



import subprocess
import time
from typing import Any

from autofmt.services.logger import log_error, log_success

last_formatted = {}


def run_formatters(filepath: str, config: dict[str, Any]) -> None:
    cooldown = config.get("cooldown_seconds", 1.0)
    now = time.time()
    if filepath in last_formatted and (now - last_formatted[filepath]) < cooldown:
        return

    time.sleep(0.5)  # Wait for file save to complete
    try:
        result = subprocess.run(
            ["python3", "-m", "py_compile", filepath],
            check=True,
            capture_output=True,
            text=True,
        )
        if result.stdout:
            print(f"\nSyntax error logsx:\n  {filepath}\n  {result.stdout.strip()}")

        if "ruff" in config["formatters"]:
            # Run fuff and capture the output
            result = subprocess.run(
                ["ruff", "check", "--fix", filepath],
                check=True,
                capture_output=True,
                text=True,
            )
            if result.stdout:
                print(f"\nRuff logs:\n  {filepath}\n  {result.stdout.strip()}")
        if "black" in config["formatters"]:
            # Run black and capture the output
            result = subprocess.run(
                ["black", filepath],
                check=True,
                capture_output=True,
                text=True,
            )
            if result.stdout:
                print(f"\nBlack logs:\n  {filepath}\n  {result.stdout.strip()}")
        last_formatted[filepath] = now
        log_success(f"\nFormatted path:\n  {filepath}")
    except subprocess.CalledProcessError:
        log_error(f"\nSyntax error logs:\n  {filepath}\n  skipping formatting.")



from os import path
from typing import Any


def get_relative_path(absolute: str, config: dict[str, Any]) -> str:
    """Get the relative path of a file from the watch path specified in the configuration."""
    return path.relpath(absolute, config.get("watch_path", "."))



import sys
from autofmt.configurations.load_config import load_config
from autofmt.services.logger import log_info
from autofmt.services.watcher import run_watcher


def main() -> None:
    config = load_config()
    log_info("Starting autofmt watcher...")
    log_info(f"At: {config['watch_path']}")
    sys.stdout.write("\033[?25l")
    sys.stdout.flush()
    run_watcher(config)


if __name__ == "__main__":
    main()



import tomllib
from typing import Any


def load_config() -> dict[str, Any]:
    """Load the configuration from pyproject.toml."""
    default = {
        "watch_path": ".",
        "formatters": ["black", "ruff"],
        "cooldown_seconds": 1.0,
    }
    try:
        with open("pyproject.toml", "rb") as f:
            pyproject = tomllib.load(f)
            return {
                **default,
                **pyproject.get("tool", {}).get("autofmt", {}),
            }
    except FileNotFoundError:
        return default
